@@ -200,15 +200,6 @@ class LinuxDevice(object):
             self.psession = self.login()
             self.connected = True
 
-            # It's a bug workaround for the issues/669:
-            # https://github.com/pexpect/pexpect/issues/669
-            LinuxDevice.cmd(self,
-                'echo "set enable-bracketed-paste off" > .netdevice_workaround_for_pexpect_issue669',
-                log_level = self.LOG_WARNING)
-            LinuxDevice.cmd(self,
-                    'export INPUTRC=$PWD/.netdevice_workaround_for_pexpect_issue669',
-                    log_level = self.LOG_WARNING)
-
             self.name = LinuxDevice.cmd(self, "hostname", log_level = self.LOG_WARNING).strip()
             if self.name:
                 self["name"] = self["name"] and self["name"] or self.name.split('.')[0]
@@ -236,7 +227,7 @@ class LinuxDevice(object):
             sys._getframe().f_code.co_name,
             self.version,
             time.time() - self.start_time))
-        self.cmd('rm $INPUTRC', log_level = self.LOG_WARNING)
+        #self.cmd('rm $INPUTRC', log_level = self.LOG_WARNING)
         #self.psession.close(force=True)
 
     def login(self, terminal_type = 'ansi', login_timeout = 10):
@@ -461,6 +452,8 @@ class LinuxDevice(object):
         len1 = self.psession.before.decode("utf-8").find("\r\n")
         len2 = self.psession.before.decode("utf-8").rfind("\r\n")
         output = self.psession.before[len1:len2].decode("utf-8").strip("\r\n")
+        # It's a bug workaround for the issues/669:
+        output = re.sub(r"\x1b\[\?2004l\r", "", output)
 
         if kwargs.get("redirect", False):
             # If redirect option is set then don't log to stdout
@@ -562,7 +555,7 @@ class LinuxDevice(object):
 
         count = int(timeout * 10)
         ratio = float(total) / count
-        self.log("sleep %d seconds" %(timeout))
+        self.log("%s %d seconds" %(description, timeout))
         # You can use this flag to run as a thread
         self.terminate_loop = False
         for i in range(1, count + 1):
