@@ -30,7 +30,7 @@ class OvsHost(linux.LinuxDevice):
         '''
 
         linux.LinuxDevice.__init__(self, server, **kwargs)
-        self.device = []
+        self.macs = []
 
         # start the docker if it's not running
         num = self.cmd('ps -ef | grep -w ovs-vswitchd | grep -v grep | wc -l')
@@ -157,7 +157,6 @@ class OvsHost(linux.LinuxDevice):
         # Add remote-device and it's peer self-port
         for arg in args:
             # allocate and add the device into the list
-            self.device.append(arg["name"])
 
             dtype = arg.get("type", None)
             if dtype == "container":
@@ -168,21 +167,24 @@ class OvsHost(linux.LinuxDevice):
                     %(arg["name"], arg["interface"]), log_level = 4)
                 # be compatible with the old version.
                 arg["mac"] = macaddr.split()[1]
+                self.macs.append(arg["mac"])
                 arg["ofport"] = self.ovs_get_container_ofport(arg["name"])
+
+                # Configure offlow for the same subnet
+                if self["fail-mode"] == "secure":
+                    ipaddr = ipaddress.ip_interface(arg["ip"])
+                    self.ofctl_add_flow(bridge_name,
+                        "priority=65535,arp,arp_tpa=%s actions=output:%s"
+                        %(ipaddr.ip, arg["ofport"]),
+                        "priority=60000,ip,nw_src=%s,nw_dst=%s actions=output:%s" 
+                        %(ipaddr.network, ipaddr.ip, arg["ofport"]))
+
             elif dtype == "vm":
                 self.__add_vm(bridge_name, **arg)
-                arg["ofport"] = self.ovs_get_interface("to_%s" %(arg["name"]), "ofport")
+
             else:
                 self.log("device type %s is not supported now..." %(dtype))
 
-            # Configure offlow for the same subnet
-            if self["fail-mode"] == "secure":
-                ipaddr = ipaddress.ip_interface(arg["ip"])
-                self.ofctl_add_flow(bridge_name,
-                    "priority=65535,arp,arp_tpa=%s actions=output:%s"
-                    %(ipaddr.ip, arg["ofport"]),
-                    "priority=60000,ip,nw_src=%s,nw_dst=%s actions=output:%s" 
-                    %(ipaddr.network, ipaddr.ip, arg["ofport"]))
 
     def __add_container (self, bridge_name,
             name = "con1",
@@ -216,7 +218,7 @@ class OvsHost(linux.LinuxDevice):
                     %(ovs_get_container_port(bridge_name, name = name),
                         kwargs["vlan"]))
 
-    def __add_vm (self, bridge_name, name, type = "vm", **kwargs):
+    def __add_vm3 (self, bridge_name, name, type = "vm", **kwargs):
         '''
         '''
         p = {"name": "to_%s" %(name),
@@ -227,89 +229,104 @@ class OvsHost(linux.LinuxDevice):
             # start the vm if it doesn't auto start
             self.cmd(kwargs["start_cmd"])
 
-    def __add_vm2 (self, mask = None,
-            type = "qemu-system-x86_64",
-            name = "vm1",
-            cpu = "host", 
-            smp = "sockets=1,cores=1",
-            m = "1024M", 
-            hda = None,
-            boot = "c",
-            net = "none",
-            pidfile = "/tmp/vm1.pid",
-            monitor = "unix:/tmp/vm1monitor,server,nowait",
-            object = "memory-backend-file,id=mem,size=1024M,mem-path=/dev/hugepages,share=on",
-            numa = "node,memdev=mem",
-            chardev = None,
-            netdev = None,
-            device = None,
-            **kwargs):
+    def __add_vm (self, bridge_name, **kwargs):
         '''
-        taskset 8 qemu-system-x86_64 -name $VM_NAME -cpu host -smp sockets=1,cores=1 -m 1024M -hda $QCOW2_IMAGE -boot c -enable-kvm -net none -no-reboot -pidfile /tmp/vm1.pid -monitor unix:/tmp/vm1monitor,server,nowait -object memory-backend-file,id=mem,size=1024M,mem-path=/dev/hugepages,share=on -numa node,memdev=mem -mem-prealloc -chardev socket,id=char1,path=$VHOST_SERVER_PATH,server -netdev type=vhost-user,id=net1,chardev=char1,vhostforce -device virtio-net-pci,netdev=net1,mac=00:00:00:00:00:01,csum=off,gso=off,guest_tso4=off,guest_tso6=off,guest_ecn=off,mrg_rxbuf=off
+        # The following parameter are mandatory:
+        vm11 = {"name": "vm11", "type": "vm", "password": "sangfor",
+                "-hda": "/media/qemu/vm1_CentOS8.img",
+                "-vnc": ":11",
+                "-smp": "sockets=1,cores=1",
+                "-m": "4096M",
+                "port": [vm11_port0, vm11_port1]}
         '''
-        cmd = mask and ("taskset %s %s" %(mask, type)) or type
-        cmd += " -name %s -cpu %s -smp %s -m %s -hda %s -boot %s"\
-                %(name, cpu, smp, m, hda, boot)
-        if kwargs.get("enable-kvm", True):
-            cmd += " -enable-kvm"
-        if kwargs.get("no-reboot", True):
-            cmd += " -no-reboot"
-        if kwargs.get("mem-prealloc", True):
-            cmd += " -mem-prealloc"
-        cmd += " -net %s -pidfile %s -monitor %s -object %s -numa %s"\
-                %(net, pidfile, monitor, object, numa)
 
-        if (device):
-            # if the chardev/netdev/device are given, use them
-            chardev = isinstance(chardev, list) and chardev or [chardev]
-            netdev = isinstance(netdev, list) and netdev or [netdev]
-            device = isinstance(device, list) and device or [device]
-            all_chardev = {}
-            all_netdev = {}
-            for c in chardev:
-                # Parse the chardev to find its type.
-                tmp_dic = {}
-                for a in c.split(",", 1):
-                    k,v = ("=" in a) and a.split("=", 1) or (a, True)
-                    tmp_dic[k.strip()] = (isinstance(v, str)) and v.strip() or v
-                all_chardev[tmp_dic["id"]] = "-chardev %s" %(c)
-            #print(all_chardev)
-            for n in netdev:
-                # Parse the netdev to find its type.
-                tmp_dic = {}
-                for a in n.split(",", 1):
-                    k,v = ("=" in a) and a.split("=", 1) or (a, True)
-                    tmp_dic[k.strip()] = (isinstance(v, str)) and v.strip() or v
-                if ("chardev" in tmp_dic):
-                    all_netdev[tmp_dic["id"]] = " %s -netdev %s"\
-                            %(all_chardev[tmp_dic["chardev"]], n)
-                else:
-                    all_netdev[tmp_dic["id"]] = "-netdev %s" %(n)
-            #print(all_netdev)
-            for d in device:
-                # Parse the device to find its type.
-                tmp_dic = {}
-                for a in d.split(",", 1):
-                    k,v = ("=" in a) and a.split("=", 1) or (a, True)
-                    tmp_dic[k.strip()] = (isinstance(v, str)) and v.strip() or v
-                cmd += " %s -device %s" %(all_netdev[tmp_dic["netdev"]], d)
-        print(cmd)
+        kwargs["-name"] = kwargs.get("-name", kwargs.get("name", "tmp"))
+        kwargs["-cpu"] = kwargs.get("-cpu", "host")
+        kwargs["-m"] = kwargs.get("-m", "2048M")
+        kwargs["-smp"] = kwargs.get("-smp", "sockets=1,cores=1")
+        kwargs["-boot"] = kwargs.get("-boot", "c")
+        kwargs["-pidfile"] = kwargs.get("-pidfile",
+                "/tmp/%s.pid" %(kwargs["-name"]))
+        kwargs["-enable-kvm"] = kwargs.get("-enable-kvm", "")
+        kwargs["-object"] = kwargs.get("-object",
+        "memory-backend-file,id=mem,size=%s,mem-path=/dev/hugepages,share=on" %(kwargs["-m"]))
+        kwargs["-numa"] = kwargs.get("-numa", "node,memdev=mem")
+        kwargs["-mem-prealloc"] = kwargs.get("-mem-prealloc", "")
+
+        cmd = kwargs.get("mask", None) and ("taskset %s " %(kwargs["mask"])) or ""
+        cmd += "qemu-system-x86_64"
+        for k,v in kwargs.items():
+            if k[0] == "-":
+                cmd += ' %s %s' %(k,v)
+
+        for p in kwargs.get("port", []):
+            for k,v in p.items():
+                if k[0] == "-":
+                    #bypass the parameters: name, ip
+                    cmd += ' %s %s' %(k,v)
+
+            if "vhost-user" in p["-netdev"]:
+                # type: vhostuserclient
+                chardev = {}
+                for attribute in p["-chardev"].split(","):
+                    if ("=" in attribute):
+                        k,v = attribute.split("=", 1)
+                        chardev[k.strip()] = v.strip()
+                    else:
+                        chardev[attribute] = ""
+
+                netdev = {}
+                for attribute in p["-netdev"].split(","):
+                    if ("=" in attribute):
+                        k,v = attribute.split("=", 1)
+                        netdev[k.strip()] = v.strip()
+                    else:
+                        netdev[attribute] = ""
+
+                device = {}
+                for attribute in p["-device"].split(","):
+                    if ("=" in attribute):
+                        k,v = attribute.split("=", 1)
+                        device[k.strip()] = v.strip()
+                    else:
+                        device[attribute] = ""
+
+                #print("chardev: %s" %(chardev))
+                #print("netdev: %s" %(netdev))
+                #print("device: %s" %(device))
+                p["mac"] = device["mac"] # Record the mac for use later
+                port_name = "to_%s_%s" %(kwargs.get("name", "tmp"), netdev["id"])
+                ovs_port = {"name": port_name,
+                        "type": "dpdkvhostuserclient",
+                        "options:vhost-server-path": chardev["path"]}
+                self.add_port(bridge_name, **ovs_port)
+                self.macs.append(p["mac"])
+                # Configure offlow for the same subnet
+                if self["fail-mode"] == "secure":
+                    p["ofport"] = self.ovs_get_interface(port_name, "ofport")
+                    ipaddr = ipaddress.ip_interface(p["ip"])
+                    self.ofctl_add_flow(bridge_name,
+                        "priority=65535,arp,arp_tpa=%s actions=output:%s"
+                        %(ipaddr.ip, p["ofport"]),
+                        "priority=60000,ip,nw_src=%s,nw_dst=%s actions=output:%s" 
+                        %(ipaddr.network, ipaddr.ip, p["ofport"]))
+
+        tmp = self.cmd("%s -daemonize" %(cmd))
         #val = os.system(cmd + "&")
-        #self.cmd("whoami")
-        #self.cmd(cmd)
 
     def add_gateway (self, bridge_name, gw, *args):
         '''
         Add some flows to ofproto
         '''
         vxlan_port = None # Get the vxlan at first.
+
         for d in args:
-            if d["type"] == "vxlan":
+            if d.get("type", None) == "vxlan":
                 vxlan_port = self.ovs_get_interface(d["name"], "ofport")
                 break
         for d in args:
-            if d["type"] != "vxlan":
-                if ((d["name"] not in self.device) and (not vxlan_port)):
+            if d.get("type", None) != "vxlan":
+                if ((d["mac"] not in self.macs) and (not vxlan_port)):
                     # d is not on host, but there is no vxlan given.
                     self.log("Error: there is no vxlan to remote device: %s!"
                             %(d["name"]), bg_color = "red")
@@ -318,7 +335,7 @@ class OvsHost(linux.LinuxDevice):
                     "priority=10,ip,nw_dst=%s,action=mod_dl_src:%s,"
                     "mod_dl_dst:%s,dec_ttl,output:%s"
                     %(d["ip"].split("/")[0], gw["mac"], d["mac"],
-                        (d["name"] in self.device) and d["ofport"] or vxlan_port))
+                        (d["mac"] in self.macs) and d["ofport"] or vxlan_port))
 
     def add_vtep (self, bridge_name, vtep, *args):
         '''
@@ -430,23 +447,30 @@ class OvsHost(linux.LinuxDevice):
         '''
         Add some flows to ofproto
         '''
+        dstip = ipaddress.ip_interface(dst["ip"])
         if src["type"] == "container":
-            result = self.cmd('docker exec -it %s ping %s -c 1' %(src["name"],
-                dst["ip"].split("/")[0]))
+            srcip = ipaddress.ip_interface(src["ip"])
+            result = self.cmd('docker exec -it %s ping %s -c 1'
+                    %(src["name"], dstip.ip))
+        elif src["type"] == "vm":
+            srcip = ipaddress.ip_interface(src["port"][1]["ip"])
+            if dstip.ip not in srcip.network:
+                # warning to configure the route.
+                self.log("WARNING: don't forget to configure route to %s!"
+                        %(dstip.ip), bg_color = "purple")
+            result = self.cmd('sshpass -p %s ssh %s@%s ping -c 1 %s'
+                %(src["password"], src["username"],
+                    src["port"][0]["ip"].split("/")[0], dstip.ip))
         else:
-            result = None
+            return False
 
         if "received, 0% packet loss," in result:
-            self.log("PASS: %s ping %s, %s -> %s!" %(src["name"],
-                dst["name"], src["ip"].split("/")[0],
-                dst["ip"].split("/")[0]),
-                bg_color = "green")
+            self.log("PASS: %s ping %s, %s -> %s!" %(src["name"], dst["name"],
+                srcip.ip, dstip.ip), bg_color = "green")
             return True
         else:
-            self.log("FAIL: %s ping %s, %s -> %s!" %(src["name"],
-                dst["name"], src["ip"].split("/")[0],
-                dst["ip"].split("/")[0]),
-                bg_color = "red")
+            self.log("FAIL: %s ping %s, %s -> %s!" %(src["name"], dst["name"],
+                srcip.ip, dstip.ip), bg_color = "red")
             return False
 
 if __name__ == '__main__':
